/**
 * Copyright (c) 2023-present, https://github.com/ThiagoSGomes-Dev
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */



"use strict";

const chokidar = require("chokidar");
const { touch } = require("shelljs");

function FileWatcherPlugin(options) {
    this.options = options;
}

FileWatcherPlugin.prototype.apply = function (compiler) {
    if (!this.options) {
        return;
    }

    const options = this.options;
    const touchFile = options.touchFile;
    const forceBuild = function (touchFileName) {
        // -c to not create a file if one doesn't already exist.
        // Remember this file needs to be watched by WebPack, thus it should already exist.
        touch("-c", touchFileName);
    };

    compiler.hooks.done.tap("FileWatcherPlugin", function (compilation) {
        var watcher = chokidar.watch(options.watchFileGlobs, {
            persistent: options.persistance || true,
            ignored: options.ignored || false,
            ignoreInitial: options.ignoreInitial || false,
            followSymlinks: options.followSymlinks || true,
            cwd: options.cwd || ".",
            disableGlobbing: options.disableGlobbing || false,
            usePolling: options.usePolling || false,
            interval: options.interval || 100,
            binaryInterval: options.binaryInterval || 300,
            alwaysStat: options.alwaysStat || false,
            depth: options.depth || 99,
            awaitWriteFinish: {
                stabilityThreshold: options.stabilityThreshold || 250,
                pollInterval: options.pollInterval || 100,
            },

            ignorePermissionErrors: options.ignorePermissionErrors || false,
            atomic: options.atomic || true,
        });

        watcher
            .on(
                "add",
                options.onAddCallback ||
                    function (path) {
                        //forceBuild(touchFile); // causes infinite loops for "add"
                        return null;
                    }
            )
            .on(
                "change",
                options.onChangeCallback ||
                    function (path) {
                        console.log(`\n\n FileWatcherPlugin CHANGE for: ${path} \n\n`);
                        forceBuild(touchFile);
                    }
            )
            .on(
                "unlink",
                options.onUnlinkCallback ||
                    function (path) {
                        // console.log(`\n\n FileWatcherPlugin UNLINK for: ${path} \n\n`);
                        forceBuild(touchFile);
                    }
            );

        watcher
            .on(
                "addDir",
                options.onAddDirCallback ||
                    function (path) {
                        // console.log(`\n\n FileWatcherPlugin ADDDIR for: ${path} \n\n`);
                        forceBuild(touchFile);
                    }
            )
            .on(
                "unlinkDir",
                options.unlinkDirCallback ||
                    function (path) {
                        // console.log(`\n\n FileWatcherPlugin UNLINKDIR for: ${path} \n\n`);
                        forceBuild(touchFile);
                    }
            )
            .on(
                "error",
                options.onErrorCallback ||
                    function (error) {
                        // console.log(`FileWatcherPlugin error: ${error}`);
                        return null;
                    }
            )
            .on(
                "ready",
                options.onReadyCallback ||
                    function () {
                        console.log("Watching for changes in the Public folder." /*, watcher.getWatched()*/);
                    }
            )
            .on(
                "raw",
                options.onRawCallback ||
                    function (event, path, details) {
                        return null;
                    }
            );
    });
};

module.exports = FileWatcherPlugin;
